'use strict';

chrome.storage.local.set({ 'snapADDYSidebar': false });

function tabHasContentScripts(tabId, callback) {
  var result = tabIdsLoaded.indexOf(tabId) > -1;
  callback(result);
}

chrome.storage.local.get('enableShortcut', function (e) {
  if (e.enableShortcut === undefined) {
    chrome.storage.local.set({
      'enableShortcut': true
    });
  }
});

chrome.storage.local.get('enableCrawlButton', function (e) {
  if (e.enableCrawlButton === undefined) {
    chrome.storage.local.set({
      'enableCrawlButton': false
    });
  }
});

chrome.windows.onRemoved.addListener((windowId, filters) => {
  if (windowId === popupWindowId) {
    if (popupWindowId > -1) {
      popupOpeningInProgress = false;
    }
    popupWindowId = -1;
  }
});

// Remember popupWindowId to keep it as a singleton
var popupWindowId = -1;
// Remember the tabId which contains the currently opened sidebar
var sidebarTabId = -2;
// Queue all parse requests while popup is not yet openend
var queuedSnapRequests = [];
// Flag to mark that the popup is in the progress of opening
var popupOpeningInProgress = false;
// Array of Tabs where the sidebar content-script has been loaded
var tabIdsLoaded = [];

function updateWindow(windowid, left, top) {
  chrome.windows.update(windowid, {
    left: left,
    top: top,
    focused: true
  }, function () { });
}

function openAlert(alertType) {
  var url = '/alert_dist/';
  switch (alertType) {
    case 'grab':
      url += 'alertGrab.html';
      break;
    case 'open':
      url += 'alertOpenSidebar.html';
      break;
    default:
      return;
  }

  chrome.tabs.query({ active: true, currentWindow: false }, function (tabs) {
    var tab = tabs[0];
    chrome.windows.get(tab.windowId, {}, function (getwindow) {
      if (getwindow && getwindow.type === 'normal') {
        var topOffset = getwindow.top;
        var leftOffset = getwindow.left;

        var windowSize = {
          width: 440,
          height: 150,
        };

        var windowLocation = {
          left: (leftOffset + (window.screen.availWidth / 2)) - (windowSize.width / 2),
          top: (topOffset + (window.screen.availHeight / 2)) - (windowSize.height / 2)
        };

        var options = {
          url: chrome.extension.getURL(url),
          left: 0, //passing windowLocation.left here sadly has no effect so we need to refresh the position after creating it
          top: 0, //passing windowLocation.top here sadly has no effect so we need to refresh the position after creating it
          width: windowSize.width,
          height: windowSize.height,
          type: 'popup'
        };
        chrome.windows.create(options, function (window) {
          //updating the position bc. it does not work when trying to initialize it at this position
          updateWindow(window.id, windowLocation.left, windowLocation.left);
        });
      }
    });
  });
}

// True if sidebar has been hidden with the slide-toggle-button
var sidebarHidden = false;
// Do not start auto-crawl for these websites on toolbar button press

/**
 * 
 * @param {chrome.windows.CreateData} createData 
 * @returns {Promise<string>} id of the created popup window
 */
function openPopup(createData = {}) {
  const defaultData = {
    url: chrome.extension.getURL('/popup_dist/index.html'),
    width: 640,
    height: 780,
    type: 'popup'
  };

  const data = Object.assign({}, defaultData, createData);

  return new Promise(resolve => {
    // trigger open of popup
    chrome.windows.create(data, popupWindow => {
      popupWindowId = popupWindow.id;
      resolve(popupWindowId);
    });
  });
}

function closePopup() {
  if (!popupWindowId) {
    return Promise.resolve();
  }

  return new Promise(resolve => {
    chrome.windows.remove(popupWindowId, resolve)
  });
}

function pushRequestToPopup(request) {
  console.log('PUSH TO POPUP');
  console.log(request);
  console.log('Current Popup Id: ' + popupWindowId);
  console.log('Sidebar Tab Id: ' + sidebarTabId);
  console.log('Current Tab Id: ' + currentTabId);
  console.log('Opening in progress: ' + popupOpeningInProgress);
  if ((popupWindowId < 0 && sidebarTabId < 0) || popupOpeningInProgress) {
    // queue for when popup gets opened
    queuedSnapRequests.push(request);
    // Open either new popup or sidebar
    if (!popupOpeningInProgress) {
      // popup will be openend
      popupOpeningInProgress = true;
      chrome.storage.local.get('snapADDYSidebar', function (result) {
        var sideBarMode = result.snapADDYSidebar;
        console.log('In sidebar mode ' + sideBarMode);
        // If not in sidebar mode
        if (!sideBarMode) {
          openPopup();
          // If in sidebar mode
        } else {
          tabHasContentScripts(currentTabId, function (hasContentScripts) {
            if (hasContentScripts) {
              console.log('Has content scripts');
              sidebarTabId = currentTabId;
              chrome.tabs.sendMessage(currentTabId, { toggleSidebar: true });
            } else {
              popupOpeningInProgress = false;
              openAlert('open');
            }
          });
        }
      });
    }
    // Else just message the existing one
  } else {
    var openSidebarFirst = false;
    // Send Message
    if (popupWindowId < 0) {
      if (sidebarHidden) {
        openSidebarFirst = true;
        sidebarHidden = false;
      } else if (sidebarTabId >= 0 && currentTabId >= 0 && sidebarTabId !== currentTabId) {
        // If certain requests, just send it to the open sidebar and don't bother open/closing it
        if (request.msg && (request.msg.toLowerCase().indexOf('oauth') > -1)) {
          console.log('Send directly to the already opened sidebar');
          chrome.tabs.sendMessage(sidebarTabId, request);
          // Close sidebar on other tab and open on current tab
        } else {
          tabHasContentScripts(currentTabId, function (hasContentScripts) {
            if (hasContentScripts) {
              console.log('First close/open sidebar and push request to the queue first');
              // Queue the request
              queuedSnapRequests.push(request);
              // Handle closing and opening of sidebar
              console.log('Close sidebar on tab ' + sidebarTabId);
              chrome.tabs.sendMessage(sidebarTabId, { toggleSidebar: true });
              sidebarTabId = -1;
              sidebarTabId = currentTabId;
              popupOpeningInProgress = true;
              console.log('Open sidebar on tab ' + sidebarTabId);
              chrome.tabs.sendMessage(currentTabId, { toggleSidebar: true });
            } else {
              popupOpeningInProgress = false;
              openAlert('open');
            }
          });
        }
        // If empty toSnap request, toggle sidebar
      } else if (request.toggle || sidebarTabId < 0) {
        console.log('toSnap ' + request.toSnap);
        console.log('Toggle sidebar on current page');
        chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
          var tab = tabs[0];
          currentTabId = tab.id;
          if (sidebarTabId > 0) {
            //sidebar will be closed
            sidebarTabId = -1;
          } else {
            sidebarTabId = tab.id;
          }
          chrome.tabs.sendMessage(tab.id, { toggleSidebar: true });
        });
      }
    }
    // Send the actual message
    if (sidebarTabId > 0) {
      chrome.tabs.sendMessage(sidebarTabId, { sidebarUnHide: true });
      chrome.tabs.sendMessage(sidebarTabId, request);
    } else if (popupWindowId > 0) {
      console.log('Focus window');
      chrome.runtime.sendMessage(request);
      chrome.windows.update(popupWindowId, { focused: true });
    }
  }
}

function pushSnapRequestToPopup(text, options) {
  console.log('PUSH SNAP REQUEST');
  let configOptions = {
    toSnap: text,
    fromShortcut: true
  };
  if (options) {
    configOptions = Object.assign(configOptions, options);
  }
  pushRequestToPopup(configOptions);
}

function queryContentScriptForSelection(callback) {
  console.log('QUERY CONTENT SCRIPT FOR SELECTION');
  chrome.tabs.query({
    active: true,
    currentWindow: true
  }, function (tabs) {
    console.log('SEND MESSAGE TO CURRENT TAB');
    chrome.tabs.sendMessage(tabs[0].id, {
      getSelection: true
    }, function (response) {
      console.log('RECEIVE ANSWER FROM CURREN TAB');
      if (response && response.inProgress) {
        callback('');
      } else {
        if (!response) {
          console.warn('This page does not support content scripts');
          callback('');
        }
      }
    });
  });
}

// Hook up context menu
chrome.contextMenus.create({
  title: chrome.i18n.getMessage('contextMenuGrabLabel'),
  contexts: ['selection'],
  onclick: function (event) {
    //TODO: Use contentscript instead of selectionText to retrieve linebreaks etc
    //http://stackoverflow.com/questions/9996540/recognize-multple-lines-on-info-selectiontext-from-context-menu?lq=1
    tabHasContentScripts(currentTabId, function (hasContentScripts) {
      if (hasContentScripts) {
        queryContentScriptForSelection(function (selection) {
          pushSnapRequestToPopup(selection);
        });
      } else {
        openAlert('grab');
      }
    });
  }
});

chrome.webNavigation.onHistoryStateUpdated.addListener(function (details) {
  console.log('details');
  console.log(details);
  if (new RegExp('https://(.*).lightning.force.com/(.*)').test(details.url)) {
    console.log('MATCH');
    chrome.tabs.executeScript(null, { file: "/scripts/updaterButtons/salesforce-lightning.js" });
  }
});

// Listen to messages
chrome.runtime.onMessage.addListener(
  function (request, sender, sendResponse) {
    // If snap request is incoming
    if (request.snapMe) {
      console.log('INCOMING SNAP REQUEST');
      pushSnapRequestToPopup(request.snapMe);
    }
    // If openPopup Event is incoming
    if (request.openPopup) {
      pushSnapRequestToPopup('');
    }

    // If switchToPage Event is incoming
    if (request.openPage) {
      pushSnapRequestToPopup('', { msg: 'openPage', page: request.page });
    }

    // If popup is ready event is incoming
    if (request.newPopupReady) {
      console.log('new popup ready event received');
      // reset flag
      popupOpeningInProgress = false;
      // handle any queued snap requests
      queuedSnapRequests.forEach(function (request) {
        console.log('Flush request ' + request);
        console.log(request);
        // pushSnapRequestToPopup(toSnap);
        // Send Message
        if (popupWindowId > 0) {
          chrome.runtime.sendMessage(request);
        } else if (sidebarTabId > 0) {
          chrome.tabs.sendMessage(sidebarTabId, request);
        }
      });
      if (popupWindowId > 0) {
        chrome.windows.update(popupWindowId, { focused: true });
      }
      //clear queue
      queuedSnapRequests = [];
    }
    if (request.leads) { // Leads request
      pushRequestToPopup(request.data);
    }
    if (request.crawl) { // crawl request
      pushRequestToPopup(request.data);
    }
    if (request.oauth) {
      pushRequestToPopup(request.data);
    }

    if (request.toggleSidebarMode) {
      if (popupWindowId < 0) {
        chrome.storage.local.set({ 'snapADDYSidebar': false }, function () {
          console.log('SET SIDEBARMODE TO FALSE');
          if (sidebarTabId > -1) {
            chrome.tabs.sendMessage(sidebarTabId, { toggleSidebar: true });
          }
          sidebarTabId = -1;
          pushRequestToPopup('');
        });
      } else {
        tabHasContentScripts(currentTabId, function (hasContentScripts) {
          if (hasContentScripts) {
            chrome.storage.local.set({ 'snapADDYSidebar': true }, function () {
              console.log('SET SIDEBARMODE TO TRUE');
              chrome.windows.remove(popupWindowId);
              sidebarTabId = currentTabId;
              chrome.tabs.sendMessage(currentTabId, { toggleSidebar: true });
              chrome.tabs.update(currentTabId, {});
            });
          } else {
            openAlert('open');
          }
        });
      }
    }

    if (request.contentScriptLoaded) {
      var tabId = sender.tab.id;
      tabIdsLoaded.push(tabId);
    }

    if (request.close) {
      chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
        chrome.tabs.remove(tabs[0].id);
      });
    }

    // visitreport export button
    if (request.visitReport) {
      pushRequestToPopup(request.data);
    }

    // notify all tabs that the export is done
    if (request.visitReportDone) {
      chrome.tabs.query({}, function (tabs) {
        for (var i in tabs) { chrome.tabs.sendMessage(tabs[i].id, request); }
      });
    }

    // sidebar click on toggle button
    if (request.sidebarHidden) {
      sidebarHidden = true;
    }

    // oldschool updater -> google image search
    if (request.openPopupWindow) {
      window.chrome.windows.create({
        url: request.openPopupWindow.url,
        type: 'popup',
        state: 'maximized'
      });
    }

    // ChromeAPIService -> Google Image Search
    if (request.openWindow) {
      window.chrome.windows.create({
        url: request.openWindow.url,
        type: 'normal',
        state: 'maximized'
      });
    }

    // Sidebar needs to reroute HTTP Requests through background script (CORS)
    if (request.rerouteHTTP) {
      var requestData = request.rerouteHTTP;
      if (requestData.url !== null && requestData.url !== undefined && requestData.method !== null && requestData.method !== undefined) {
        var newRequest = new XMLHttpRequest();
        var data = requestData.data;

        newRequest.open(requestData.method, requestData.url, true);

        if (requestData.contentType) {
          newRequest.setRequestHeader('Content-Type', requestData.contentType);
        }

        if (requestData._headers) {
          Object.keys(requestData._headers).forEach(function (key) {
            newRequest.setRequestHeader(key, requestData._headers[key]);
          });
        }

        newRequest.onreadystatechange = function () {
          if (newRequest.readyState === XMLHttpRequest.DONE) {
            var result = newRequest.responseText;
            sendResponse({ data: result, status: newRequest.status });
          }
        };

        if (requestData.data) {
          newRequest.send(data);
        } else {
          newRequest.send();
        }

        return true;
      }
    }

    // Google Image search
    // close google after selecting an image
    if (request.closeTab) {
      console.log('closing');
      console.log(request);
      var senderTabId = sender.tab.id;
      console.log('sender-tab-id:' + senderTabId);
      console.log('currentTabId: ' + currentTabId);
      console.log('sidebarTabId: ' + sidebarTabId);
      chrome.tabs.remove(senderTabId);
    }

    // Oldschool Updater -> refresh crm after save
    if (request.refreshPage) {
      chrome.tabs.query({}, function (tabs) {
        for (var i in tabs) {

          var toCheck = null;
          if (request.url) { toCheck = request.url; }
          if (request.entityId) { toCheck = request.entityId.substr(0, 5); }

          if (tabs[i].url && tabs[i].url.indexOf(toCheck) > -1) {
            chrome.tabs.reload(tabs[i].id);
          }
        }
      });
    }



    switch (request.msg) {

      // used when switching from popup to "normal window"
      // see msg = 'open_in_normal_window'
      case 'close_popup':
        closePopup();
        break;

      // crm updater button
      case 'integration_update_to_grabber':
        pushRequestToPopup({ msg: 'integration_update', data: request.data });
        break;

      // needed for firefox in order to disable social tracking
      // see: https://github.com/snapADDY/snapaddy-browser-popup/issues/3396#issuecomment-549781433
      case 'open_in_normal_window':
        openPopup({ type: 'normal' });
        break;

      case 'pushImage':
        if (sidebarTabId > 0) {
          // send message only to sidebar, the popup window has it's own pushImage listener
          chrome.tabs.sendMessage(sidebarTabId, { msg: 'pushImage', imgRef: request.imgRef });
        }
        break;
    }

  });

// Listen to browser action button click
// Grabber will allways start in sidebar mode right now
chrome.browserAction.onClicked.addListener(function () {
  console.log('BROWSER ACTION CLICKED');
  console.log('Current Popup Id: ' + popupWindowId);
  console.log('Sidebar Tab Id: ' + sidebarTabId);
  console.log('Current Tab Id: ' + currentTabId);
  console.log('Opening in progress: ' + popupOpeningInProgress);
  pushRequestToPopup({ toggle: true });
});

// Listen if browser popup got closed
chrome.windows.onRemoved.addListener(function (removedWindowId) {
  if (popupWindowId === removedWindowId) {
    popupWindowId = -1;
  }
});

//actually: last focused tab in a normal window (no popup, app, devtools etc.)
var currentTabId = -1;

chrome.tabs.onActivated.addListener(function (newTab) {
  try {
    chrome.windows.get(newTab.windowId, { /*windowTypes: ['normal']*/ }, function (window) {
      if (window && window.type === 'normal') {
        currentTabId = newTab.tabId;
      }
    });
  } catch (e) {
    console.log(e);
  }
});

chrome.tabs.onUpdated.addListener(function (tabId) {
  // Sidebar disappears on navigation
  if (sidebarTabId === tabId) {
    // Check if sidebar was closed by the update
    chrome.tabs.sendMessage(tabId, { isSidebarVisible: true }, function (isSidebarVisible) {
      if (!isSidebarVisible) {
        console.log('Sidebar no longer visible after tab update -> Update state accordingly');
        sidebarTabId = -1;
        popupOpeningInProgress = false;
      } else {
        console.log('Sidebar still visible after tab update -> Everything is fine');
      }
    });
  }
});

chrome.tabs.onRemoved.addListener(function (tabId) {
  // Sidebar disappears on navigation
  if (sidebarTabId === tabId) {
    sidebarTabId = -1;
    popupOpeningInProgress = false;
  }
});

chrome.windows.onFocusChanged.addListener(function (windowId) {
  if (windowId >= 0 && windowId !== popupWindowId) {
    try {
      chrome.windows.get(windowId, { populate: true }, function (window) {
        if (window && window.type === 'normal') {
          window.tabs.forEach(function (tab) {
            if (tab.selected) {
              currentTabId = tab.id;
            }
          });
        }
      });
    } catch (e) {
      console.log(e);
    }
  }
});

function sendEmail(mailToUrl) {
  var emailUrl = mailToUrl;
  chrome.tabs.create({
    url: emailUrl
  }, function (tab) {
    setTimeout(function () {
      chrome.tabs.remove(tab.id);
    }, 400);
  });
}

const isFirefox = window.location.href.includes('moz-extension');
const isChrome = window.location.href.includes('chrome-extension');

const onBeforeSendHeadersExtraParams = ['requestHeaders', 'blocking'];
if (isChrome) {
  // hacky solution to get the chrome version (apparently there is no api for that)
  const version = parseInt((/Chrome\/([0-9]+)/.exec(navigator.userAgent) || [, 0])[1]);

  if (version >= 72) {
    // from chrome >=72 we need the "extraHeaders" parameter to read the Cookie Header
    // see: https://developer.chrome.com/extensions/webRequest#life_cycle_footnote
    // but firefox does not support this param!
    onBeforeSendHeadersExtraParams.push('extraHeaders');
  }
}

chrome.webRequest.onBeforeSendHeaders.addListener(
  function (details) {
    var headers = details.requestHeaders;
    var blockingResponse = {};

    for (var i = 0, l = headers.length; i < l; ++i) {
      if (headers[i].name === 'Cookie') {
        var cookies = headers[i].value;
        var sessionCookiePrefix = 'JSESSIONID="';
        var sessionIdIndex = cookies.indexOf(sessionCookiePrefix);

        if (sessionIdIndex >= -1) {
          var cookieSubstr = cookies.substr(sessionIdIndex + sessionCookiePrefix.length);
          cookieSubstr = cookieSubstr.substr(0, cookieSubstr.indexOf('";'));
          if (cookieSubstr) {
            pushRequestToPopup({ linkedInCookie: cookieSubstr });
          }
        }

        break;
      }
    }

    blockingResponse.requestHeaders = headers;
    return blockingResponse;
  },
  {
    urls: ['https://www.linkedin.com/*orig=snaptoken'],
    types: ['xmlhttprequest']
  },
  onBeforeSendHeadersExtraParams
);

let cspHeaders = [
  'content-security-policy',
  'x-webkit-csp'
];

let cspSources = [
  'connect-src',
  'default-src',
  'font-src',
  'frame-src',
  'img-src',
  'media-src',
  'object-src',
  'script-src',
  'style-src'
];

function isCspHeader(headerName) {
  return cspHeaders.includes(headerName.toLowerCase());
}

chrome.webRequest.onHeadersReceived.addListener(function (details) {
  for (var i = 0, l = details.responseHeaders.length; i < l; ++i) {
    var responseHeader = details.responseHeaders[i];
    if (isCspHeader(responseHeader.name)) {
      var csp = responseHeader.value;
      for (var j = 0; j < cspSources.length; j++) {
        var cspSource = cspSources[j];
        csp = csp.replace(cspSource, cspSource + ' https://snapaddy.com https://*.snapaddy.com');
      }
      details.responseHeaders[i].value = csp;
    }
  }

  return { responseHeaders: details.responseHeaders };
},
  {
    urls: ['http://*.linkedin.com/*', 'https://*.linkedin.com/*'],
    types: ['main_frame']
  },
  ['blocking', 'responseHeaders']
);

chrome.webRequest.onBeforeRequest.addListener(
  function (details) {
    const tabId = details.tabId;
    // Firefox does not inject scripts on pages with content-type application/json.. so lets workaround!
    if (isFirefox) {
      const filter = chrome.webRequest.filterResponseData(details.requestId);
      const decoder = new TextDecoder("utf-8");
      const responses = [];

      filter.ondata = event => {
        responses.push(decoder.decode(event.data, { stream: true }));
      }
      filter.onstop = () => {
        chrome.runtime.sendMessage({ oauth: true, json: responses.join('') });
        chrome.tabs.remove(tabId);
        filter.disconnect();
      }
    } else if (isChrome) { // chrome.webRequest.filterResponseData does not exist for chrome
      function getJson(updatedTabId, changeInfo) {
        if (updatedTabId === tabId && changeInfo.status === 'complete') {
          chrome.tabs.executeScript(tabId, {
            code: '(' + function () {
              return document.body.innerText;
            } + ')();'
          }, function (result) {
            chrome.runtime.sendMessage({ oauth: true, json: result[0] });
            chrome.tabs.remove(tabId);
          });

          chrome.tabs.onUpdated.removeListener(getJson);
        }
      }

      chrome.tabs.onUpdated.addListener(getJson);
    }

    return {};
  },
  {
    urls: [
      "https://staging.snapaddy.com/api/sso/microsoft/callback*",
      "https://api.snapaddy.com/api/sso/microsoft/callback*",
      "https://staging.snapaddy.com/api/sso/google/callback*",
      "https://api.snapaddy.com/api/sso/google/callback*"
    ],
    types: ['main_frame']
  },
  ['blocking']
);
