(function () {
  'use strict';

  var getObjectIdFromURL = function () {
    const pathName = window.location.pathname || '';
    return pathName.split('/').find(Boolean) || null;
  };

  var isActive = function (entity) {
    return document.getElementsByClassName(entity + ' detailPage').length > 0;
  };

  var fillMenuList = function (options) {
    var addOption = function (option) {
      var menuList = document.querySelector('#snapaddy-menu ul');

      var li = document.createElement('li');
      li.innerHTML = option.title;
      li.id = option.id;
      li.onclick = option.callback;

      menuList.appendChild(li);
    };

    var menu = document.getElementById('snapaddy-menu');
    if (menu) {
      menu.parent.removeChild(menu);
    }

    var html = `
      <div id="snapaddy-menu">
        <div>
          <ul></ul>
        </div>
      </div>
    `;

    var btn = document.getElementById('snapaddy-button');
    btn.innerHTML = btn.innerHTML + html;

    for (var i in options) {
      addOption(options[i]);
    }
  };

  var createMenu = function (options) {
    var text = 'snapADDY Updater';

    var btn = document.getElementById('snapaddy-button');
    if (btn) {
      btn.parent.removeChild(btn);
    }

    var buttonBar = document.getElementById('topButtonRow');

    if (!buttonBar) {
      return false;
    }

    var snapaddyButton = document.createElement('div');
    snapaddyButton.className = 'btn';
    snapaddyButton.id = 'snapaddy-button';
    snapaddyButton.name = text;
    snapaddyButton.title = text;
    snapaddyButton.type = 'button';
    snapaddyButton.innerHTML = `<span class="icon"></span>${text}`;

    buttonBar.appendChild(snapaddyButton);

    fillMenuList(options);

    document.getElementById('snapaddy-button').onclick = function (e) {
      var menu = document.getElementById('snapaddy-menu');
      if (!menu) { return false; }
      if (menu.className.indexOf('visible') > -1) { // isVisible
        menu.className = menu.className.replace(/visible/g, '');
      } else {
        menu.className = menu.className + ' visible';
      }
      e.stopPropagation();
      e.preventDefault();
      return false;
    };

    document.body.onclick = function () {
      var menu = document.getElementById('snapaddy-menu');
      if (!menu) { return false; }
      menu.className = menu.className.replace(/visible/g, '');
    };

    return true;
  };


  var createLeadButtons = function () {
    const updateLeadLabel = chrome.i18n.getMessage('snapaddyUpdater_lead');
    createMenu([
      {
        title: updateLeadLabel,
        id: 'lead_update',
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'salesforce',
              specific: 'lead',
              lead_id: getObjectIdFromURL()
            }
          });
        }
      }
    ]);
  };

  var createContactButtons = function () {
    const updateContactLabel = chrome.i18n.getMessage('snapaddyUpdater_contact');
    const createContactLabel = chrome.i18n.getMessage('snapaddyUpdater_create_contact');
    createMenu([
      {
        title: updateContactLabel,
        id: 'contact_update',
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'salesforce',
              specific: 'contact',
              contact_id: getObjectIdFromURL()
            }
          });
        }
      },
      {
        title: createContactLabel,
        id: 'contact_create_contact',
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'salesforce',
              specific: 'contact_create_contact',
              contact_id: getObjectIdFromURL()
            }
          });
        }
      }
    ]);
  };

  var createAccountButtons = function () {
    const updateAccountLabel = chrome.i18n.getMessage('snapaddyUpdater_account');
    const createContactLabel = chrome.i18n.getMessage('snapaddyUpdater_create_contact');
    createMenu([
      {
        title: updateAccountLabel,
        id: 'account_update',
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'salesforce',
              specific: 'account',
              account_id: getObjectIdFromURL()
            }
          });
        }
      },
      {
        title: createContactLabel,
        id: 'account_create_contact',
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'salesforce',
              specific: 'account_create_contact',
              account_id: getObjectIdFromURL()
            }
          });
        }
      }
    ]);
  };

  var createUpdateButton = function () {
    chrome.storage.local.get('SNAPADDY_UPDATER_CONFIG', function (items) {
      let configArray = items.SNAPADDY_UPDATER_CONFIG;
      if (configArray && configArray.length > 0) {
        configArray.forEach((config) => {
          if (config.key === 'salesforce') {
            if (config.showUpdaterButton) {
              if (!config.disabledEntities.account && isActive('accountTab')) {
                createAccountButtons();
              }

              if (!config.disabledEntities.contact && isActive('contactTab')) {
                createContactButtons();
              }

              if (!config.disabledEntities.lead && isActive('leadTab')) {
                createLeadButtons();
              }
            }
          }
        });
      }
    });
  };


  window.addEventListener('load', function () {
    createUpdateButton();
  }, false);
})();
